﻿using System;
using System.Collections.Generic;

using System.Text;

namespace Syncfusion.Windows.Forms.Diagram.Samples.DiagramTool
{
    public class DiagramEx: Syncfusion.Windows.Forms.Diagram.Controls.Diagram
    {
        public bool BringSelectionToFront(bool considerLayers)
        {
            //use instead of Controller.SendToBack
            if (considerLayers)
            {
                bool flag = false;
                if (!Controller.SelectionList.IsEmpty && (this.Model != null))
                {
                    Model.BeginUpdate();
                    Model model = this.Model;
                    model.HistoryManager.StartAtomicAction("BringToFront");
                    lock (Controller.SelectionList)
                    {
                        NodeCollection nodes = new NodeCollection();
                        nodes.AddRange(Controller.SelectionList);
                        nodes.Sort(new ReverseZOrderComparer());
                        foreach (Node node in nodes)
                        {
                            if (node.Layers.Count == 0)
                            {
                                model.BringToFront(node);
                            }
                            else
                            {
                                BringNodeToFrontOfLayer(node);
                            }
                            //model.SendToBack(node);
                        }
                        flag = true;
                    }

                    model.HistoryManager.EndAtomicAction();
                    EndUpdate();
                    Invalidate(true);
                }
                return flag;

            }
            else
            {
                return Controller.SendToBack();
            }
        }

        public bool BringSelectionForward(bool considerLayers)
        {
            if (considerLayers)
            {
                bool flag = false;
                if (!Controller.SelectionList.IsEmpty && (this.Model != null))
                {
                    Model model = this.Model;
                    model.BeginUpdate();
                    model.HistoryManager.StartAtomicAction("Bring Forward");
                    lock (Controller.SelectionList)
                    {
                        NodeCollection nodes = new NodeCollection();
                        nodes.AddRange(Controller.SelectionList);
                        nodes.Sort(new ReverseZOrderComparer());
                        foreach (Node node in nodes)
                        {
                            if (node.Layers.Count == 0)
                            {
                                model.BringForward(node);
                            }
                            else
                            {
                                BringNodeForwardInLayer(node);
                            }
                            //model.SendToBack(node);
                        }
                        flag = true;
                    }
                    model.HistoryManager.EndAtomicAction();
                    EndUpdate();
                    Invalidate(true);
                }
                return flag;

            }
            else
            {
                return Controller.BringForward();
            }
        }


        public bool SendSelectionBackward(bool considerLayers)
        {
            if (considerLayers)
            {
                bool flag = false;
                Model.BeginUpdate();
                if (!Controller.SelectionList.IsEmpty && (this.Model != null))
                {
                    Model model = this.Model;
                    model.HistoryManager.StartAtomicAction("Send Backward");
                    lock (Controller.SelectionList)
                    {
                        NodeCollection nodes = new NodeCollection();
                        nodes.AddRange(Controller.SelectionList);
                        nodes.Sort(new ZOrderComparer());
                        foreach (Node node in nodes)
                        {
                            if (node.Layers.Count == 0)
                            {
                                model.SendBackward(node);
                            }
                            else
                            {
                                SendNodeBackInLayer(node);
                            }
                            //model.SendToBack(node);
                        }
                        flag = true;
                    }
                    model.HistoryManager.EndAtomicAction();
                }
                EndUpdate();
                Invalidate(true);
                return flag;

            }
            else
            {
                return Controller.SendBackward();
            }
        }

        private int BringNodeForwardInLayer(Node node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            int index = -1;

            if (node.Parent != Model)
            {
                IZOrderContainer parent = node.Parent as IZOrderContainer;
                if (parent != null)
                {
                    parent.BringForward(node);
                }
                return index;
            }
            //determine the lowest zorder in the node's current layer
            Layer layer = node.Layers[0];
            layer = Model.Layers[layer.Name];
            int maxZ = node.ZOrder;
            for (int i = 0; i < layer.Nodes.Count; i++)
            {
                if (layer.Nodes[i] != node && layer.Nodes[i].ZOrder > maxZ)
                {
                    maxZ = layer.Nodes[i].ZOrder;
                }
            }
            if (maxZ > node.ZOrder)
            {
                index = Model.BringForward(node);
            }
            return index;
        }

        private int SendNodeBackInLayer(Node node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            int index = -1;

            if (node.Parent != Model)
            {
                IZOrderContainer parent = node.Parent as IZOrderContainer;
                if (parent != null)
                {
                    parent.SendBackward(node);
                }
                return index;
            }
            //determine the lowest zorder in the node's current layer
            Layer layer = node.Layers[0];
            layer = Model.Layers[layer.Name];
            int maxZ = node.ZOrder;
            for (int i = 0; i < layer.Nodes.Count; i++)
            {
                if (layer.Nodes[i] != node && layer.Nodes[i].ZOrder < maxZ)
                {
                    maxZ = layer.Nodes[i].ZOrder;
                }
            }
            if (maxZ < node.ZOrder)
            {
                index = Model.SendBackward(node);
            }
            return index;
        }


        public bool SendSelectionToBack(bool considerLayers)
        {
            //use instead of Controller.SendToBack
            if (considerLayers)
            {
                Model.BeginUpdate();
                bool flag = false;
                if (!Controller.SelectionList.IsEmpty && (this.Model != null))
                {
                    Model model = this.Model;
                    model.HistoryManager.StartAtomicAction("SendToBack");
                    lock (Controller.SelectionList)
                    {
                        NodeCollection nodes = new NodeCollection();
                        nodes.AddRange(Controller.SelectionList);
                        nodes.Sort(new ZOrderComparer());
                        foreach (Node node in nodes)
                        {
                            if (node.Layers.Count == 0)
                            {
                                model.SendToBack(node);
                            }
                            else
                            {
                                SendNodeToBackOfLayer(node);
                            }
                            //model.SendToBack(node);
                        }
                        flag = true;
                    }
                    model.HistoryManager.EndAtomicAction();
                }

                EndUpdate();
                Invalidate(true);
                return flag;

            }
            else
            {
                return Controller.SendToBack();
            }
        }

        /// <summary>
        /// only concerned with the node's First Layer (Layers[0])
        /// Determines lowest Zindex of items in this layer
        /// sends node to that zorder
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public int SendNodeToBackOfLayer(Node node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            int index = -1;

            if (node.Parent != Model)
            {
                IZOrderContainer parent = node.Parent as IZOrderContainer;
                if (parent != null)
                {
                    parent.SendToBack(node);
                }
                return index;
            }
            //determine the lowest zorder in the node's current layer
            Layer layer = node.Layers[0];
            layer = Model.Layers[layer.Name];
            int minZindex = node.ZOrder;

            for (int i = 0; i < layer.Nodes.Count; i++)
            {
                if (layer.Nodes[i] != node && layer.Nodes[i].ZOrder < minZindex)
                {
                    minZindex = layer.Nodes[i].ZOrder;
                }
            }
            //must also determine minindex in layer above this one
            int layerindex = Model.Layers.IndexOf(layer);
            if (layerindex < Model.Layers.Count - 1)
            {
                Layer nextLayer = Model.Layers[layerindex + 1];
                for (int i = 0; i < nextLayer.Nodes.Count; i++)
                {
                    if (nextLayer.Nodes[i] != node && nextLayer.Nodes[i].ZOrder < minZindex)
                    {
                        minZindex = nextLayer.Nodes[i].ZOrder;
                    }
                }
            }

            if (minZindex < node.ZOrder)
            {
                index = Model.SetZOrder(node, minZindex);
            }

            return index;
        }

        /// <summary>
        /// only concerned with the node's First Layer (Layers[0])
        /// Determines max Zindex of items in this layer
        /// sends node to that zorder
        /// </summary>
        /// <param name="node"></param>
        /// <returns></returns>
        public int BringNodeToFrontOfLayer(Node node)
        {
            if (node == null)
            {
                throw new ArgumentNullException("node");
            }
            int index = -1;

            if (node.Parent != Model)
            {
                IZOrderContainer parent = node.Parent as IZOrderContainer;
                if (parent != null)
                {
                    parent.BringToFront(node);
                }
                return index;
            }
            //determine the lowest zorder in the node's current layer
            Layer layer = node.Layers[0];
            layer = Model.Layers[layer.Name];
            int maxZ = node.ZOrder;
            for (int i = 0; i < layer.Nodes.Count; i++)
            {
                if (layer.Nodes[i] != node && layer.Nodes[i].ZOrder > maxZ)
                {
                    maxZ = layer.Nodes[i].ZOrder;
                }
            }

            //now determine for next lowest layer just in case
            int layerIndex = Model.Layers.IndexOf(layer);
            if (layerIndex > 0)
            {
                Layer nextLayer = Model.Layers[layerIndex - 1];
                for (int i = 0; i < nextLayer.Nodes.Count; i++)
                {
                    if (nextLayer.Nodes[i] != node && nextLayer.Nodes[i].ZOrder > maxZ)
                    {
                        maxZ = nextLayer.Nodes[i].ZOrder;
                    }
                }
            }
            if (maxZ > node.ZOrder)
            {
                index = Model.SetZOrder(node, maxZ);
            }
            return index;
        }
    }
}
